# -*- coding: utf-8 -*-
from collective.campaignmonitor import _  # noqa
from collective.campaignmonitor.interfaces import ICampaignMonitorConnection
from collective.campaignmonitor.interfaces import ICampaignMonitorSettings
from collective.campaignmonitor.interfaces import INewsletterSubscribe
from plone import api
from plone.registry.interfaces import IRegistry
from plone.z3cform.fieldsets import extensible
from plone.z3cform.layout import wrap_form
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import HIDDEN_MODE
from zope.component import getUtility


class NewsletterSubscribeForm(extensible.ExtensibleForm, form.Form):
    fields = field.Fields(INewsletterSubscribe)
    ignoreContext = True
    id = "cm-subscribe-form"
    label = _(u"Subscribe Form")

    def updateActions(self):
        super(NewsletterSubscribeForm, self).updateActions()
        self.actions["subscribe"].addClass("context")

    def updateWidgets(self):
        super(NewsletterSubscribeForm, self).updateWidgets()

        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(ICampaignMonitorSettings)
        self.connection = getUtility(ICampaignMonitorConnection)

        # Retrieve the list id either from the request/form or fall back to
        # the default_list setting.
        list_id = self.context.REQUEST.get("list_id", None)
        list_id = list_id or self.request.form.get(
            "form.widgets.list_id", None
        )
        if list_id is not None:
            self.widgets["list_id"].mode = HIDDEN_MODE
            self.widgets["list_id"].value = list_id

    @button.buttonAndHandler(
        _(u"subscribe_to_newsletter_button", default=u"Subscribe"),
        name="subscribe",
    )
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        list_id = data.get("list_id")
        email = data.get("email")

        result = self.connection.subscribe(email, list_id)
        if result:
            details = self.connection.list_details(list_id)
            if details.get("ConfirmedOptIn", None):
                message = _(
                    u"Your subscription was processed correctly. Check your e-mail for a confirmation message."
                )
            else:
                message = _(u"You have been subscribed to the newsletter")
            api.portal.show_message(message, request=self.request, type="info")
        else:
            message = _(
                u"Your subscription could not be processed. Please try again."
            )
            api.portal.show_message(
                message, request=self.request, type="error"
            )

        self.request.response.redirect(self.context.absolute_url())


NewsletterSubscribeView = wrap_form(NewsletterSubscribeForm)



# -*- coding: utf-8 -*-
from collective.campaignmonitor import _
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import re


class ICollectiveCampaignmonitorLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class ICampaignMonitorSettings(Interface):
    api_key = schema.TextLine(
        title=_(u"CampaignMonitor API Key"),
        description=_(
            u"help_api_key",
            default=u"Enter in your CampaignMonitor key here (.e.g. "
            + u"'8b785dcabe4b5aa24ef84201ea7dcded-us4'). Log into "
            + u"xxxx.createsend.com, go to account settings -> API Keys "
            + u"click on Show API key and copy the API Key to this field.",
        ),
        default=u"",
        required=True,
    )

    client_id = schema.Choice(
        title=_("Client ID"),
        description=_(
            u"Campaign Monitor uses clients to group lists, invoices, ... Select the one that you want to use in Plone"
        ),
        vocabulary="collective.campaignmonitor.CampaignMonitorClientsVocabulary",
        required=False,
    )


class ICampaignMonitorConnection(Interface):
    def initialize():
        """ Load connection data from registy and prepare for serving results """

    def account():
        """ Return account information for campaign monitor """

    def clients():
        """ Return available clients at the campaign monitor account """

    def lists():
        """ Return available lists at the campaign monitor account """

    def subscribe(email, list_id):
        """ Subscribe 'email' to 'list_id' Campaign Monitor list """

    def list_details(list_id):
        """ get details about a given list """


class NotAnEmailAddress(schema.ValidationError):
    __doc__ = _(u"Invalid email address")


check_email = re.compile(r"[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+.)*[a-zA-Z]{2,4}").match


def validate_email(value):
    if not check_email(value):
        raise NotAnEmailAddress(value)
    return True


class INewsletterSubscribe(Interface):

    email = schema.TextLine(
        title=_(u"Email address"),
        description=_(u"help_email", default=u"Please enter your email address."),
        required=True,
        constraint=validate_email,
    )

    list_id = schema.Choice(
        title=_(u"List ID"),
        vocabulary="collective.campaignmonitor.CampaignMonitorListsVocabulary",
        required=False,
    )



# -*- coding: utf-8 -*-
from collective.campaignmonitor.interfaces import ICampaignMonitorConnection
from collective.campaignmonitor.interfaces import ICampaignMonitorSettings
from createsend import BadRequest
from createsend import Client
from createsend import CreateSend
from createsend import List
from createsend import Subscriber
from createsend import Unauthorized
from logging import getLogger
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import implementer


log = getLogger("CampaignMonitorConnection")


CLIENTS_CACHE_KEY = "collective.campaignmonitor.cache.clients"
LISTS_CACHE_KEY = "collective.campaignmonitor.cache.lists"


_marker = None


@implementer(ICampaignMonitorConnection)
class CampaignMonitorConnection(object):
    def __init__(self):
        self.registry = None
        self.settings = None
        self.api_key = None

    def initialize(self):
        if self.registry is None:
            self.registry = getUtility(IRegistry)
        if self.settings is None:
            self.settings = self.registry.forInterface(
                ICampaignMonitorSettings
            )

        self.api_key = self.settings.api_key

    def account(self):
        self.initialize()
        return self.api_key

    def update_cache(self):
        log.info("Updating cache...")
        self.initialize()
        if not self.settings.api_key:
            return

        clients = self._clients()
        if type(clients) is list:
            clients = tuple(clients)
            if self.registry[CLIENTS_CACHE_KEY] != clients:
                self.registry[CLIENTS_CACHE_KEY] = clients
                log.info("Updated client list.")

        lists = self._lists()
        if type(lists) is list:
            lists = tuple(lists)
            if self.registry[LISTS_CACHE_KEY] != lists:
                self.registry[LISTS_CACHE_KEY] = lists
                log.info("Update lists list")

        log.info("Updating cache finished.")

    def _clients(self):
        cs = CreateSend({"api_key": self.api_key})
        results = []
        try:
            clients = cs.clients()
            for client in clients:
                data = dict(client_id=client.ClientID, client_name=client.Name)
                results.append(data)
        except Unauthorized as e:
            log.info(str(e))

        return results

    def clients(self):
        self.initialize()
        cache = self.registry.get(CLIENTS_CACHE_KEY, _marker)
        if cache and cache is not _marker:
            return cache
        return self._clients()

    def _lists(self):
        cs = Client({"api_key": self.api_key}, self.settings.client_id)
        results = []
        try:
            lists = cs.lists()
            for listitem in lists:
                data = dict(list_id=listitem.ListID, list_name=listitem.Name)
                results.append(data)
        except Unauthorized as e:
            log.info(str(e))

        return results

    def lists(self):
        self.initialize()
        cache = self.registry.get(LISTS_CACHE_KEY, _marker)
        if cache and cache is not _marker:
            return cache
        return self._lists()

    def subscribe(self, email, list_id, name=None):
        self.initialize()
        subscriber = Subscriber({"api_key": self.api_key})
        try:
            subscriber.add(
                list_id=list_id,
                email_address=email,
                name=name or email,
                custom_fields=[],
                resubscribe=False,
                consent_to_track="yes",
            )
            return True
        except BadRequest as e:
            log.info(str(e))
            return False

    def list_details(self, list_id):
        self.initialize()
        cm_list = List({"api_key": self.api_key}, list_id=list_id)
        details = cm_list.details()
        data = {}
        data["ConfirmationSuccessPage"] = getattr(
            details, "ConfirmationSuccessPage", None
        )
        data["ConfirmedOptIn"] = getattr(details, "ConfirmedOptIn", None)
        data["ListID"] = getattr(details, "ListID", None)
        data["Title"] = getattr(details, "Title", None)
        data["UnsubscribePage"] = getattr(details, "UnsubscribePage", None)
        data["UnsubscribeSetting"] = getattr(
            details, "UnsubscribeSetting", None
        )

        return data



# -*- coding: utf-8 -*-
from collective.campaignmonitor import _
from collective.campaignmonitor.interfaces import ICampaignMonitorConnection
from collective.campaignmonitor.interfaces import ICampaignMonitorSettings
from plone.app.registry.browser import controlpanel
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form.interfaces import WidgetActionExecutionError
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import Invalid


try:
    from plone.protect.interfaces import IDisableCSRFProtection
except ImportError:
    # BBB for old plone.protect, default until at least Plone 4.3.7.
    IDisableCSRFProtection = None


class CampaignMonitorControlPanelForm(controlpanel.RegistryEditForm):
    schema = ICampaignMonitorSettings
    label = _("CampaingMonitor Settings")
    description = u""

    def update(self):
        if IDisableCSRFProtection is not None:
            alsoProvides(self.request, IDisableCSRFProtection)

        self.update_cache()
        super(CampaignMonitorControlPanelForm, self).update()

    def update_cache(self):
        connection = getUtility(ICampaignMonitorConnection)
        connection.update_cache()


class CampaignMonitorControlPanel(controlpanel.ControlPanelFormWrapper):
    form = CampaignMonitorControlPanelForm
    index = ViewPageTemplateFile("controlpanel.pt")

    def campaignmonitor_account(self):
        # if IDisableCSRFProtection is not None:
        #     alsoProvides(self.request, IDisableCSRFProtection)
        connection = getUtility(ICampaignMonitorConnection)
        try:
            return connection.account()
        except Exception as e:
            raise WidgetActionExecutionError(
                Invalid(
                    "Could not fetch account details from CampaignMonitor. Please check your CampaignMonitor API key: {0}".format(
                        e
                    )
                )
            )
